--[[ PoisonCrafter by MrInfernal  v0.63

    Can search by poison name or effect, then select a number to make
    Will purchase the required number of ingredients and then craft

    ToDo:
    Check Tradeskill Depot
    Create events to track: if out of money, if out of ingredients
    Get more money if needed from bank
    Get the ingredients to make more poisons until complete
    Check for and equip trophy. Re-equip weapon afterwards

    --Note: Refers to potions due to reusing code
]]

local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

-- set variables
Potions = require('poisons')
local numPot = '1' -- How many poisons to make
local typeAndEffects = {{type='Damage', effects={'All Class Damage', 'Rogue Damage', 'Rogue Damage + Lower Hate'}},
{type='Debuff', effects={'Decrease ATK', 'Slow', 'Slow and Decrease ATK', 'Snare'}},
{type='Elemental Dmg', effects={'Cold + Decrease Hate', 'Fire + Decrease Hate'}},
{type='Hate', effects={'Damage + Hate', 'Decrease Hate'}}
    }
local levelSelect = 1
local potLevel = {'Any', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X', 'XI', 'XII', 'XIII',
                'XIV', 'XV', 'XVI', 'XVII', 'XVIII', 'XIX', 'XX', 'XI', 'XII'}
local selectedTypeIndex = 1
local selectedEffectIndex = 1
local selectedPotionIndex = 1
local typeSelect = ''
local effectSelect = ''
local lookupResults = {}
local selectedPotion = {}
MPSkill = 0
MyCash = mq.TLO.Me.Cash() / 1000
local useCWTN = false
local useForage = false
local onlyVendor = false
local gatedToPOK = false
local useGatePot = false
BuyPot = false
MakePot = false
GoVendor = false
HaveTrophy = false
LvlChanged = false

if mq.TLO.FindItem('Vial of Swirling Smoke')() then
    useGatePot = true
end

-- Functions
local function getTrophy()
    local gearSlots = {'mainhand', 'offhand', 'ranged', 'ammo'}
    local trophyValue = 0

    for _, slot in ipairs(gearSlots) do
        local gear = mq.TLO.Me.Inventory(slot)
        if gear() ~= nil then
           if string.find(gear.Name(), 'Trophy') then
                if gear.SkillModValue() > 0 then
                    trophyValue = gear.SkillModValue() / 100
                    HaveTrophy = true
                end
            else
                HaveTrophy = false
            end
        end
    end
    return trophyValue
end

local function updateSkill()
    local skill = mq.TLO.Me.Skill('Make Poison')()
    local trophy = getTrophy()

    if trophy == 0 then
        MPSkill = skill
    else
        MPSkill = tonumber(skill + math.floor(skill * trophy))
    end
end

local function getBagData()
    local bagSlot = ''
    local bagData

    -- Find medicine bag, return the bagslot number and bag object
    local packSlots = {
        {slot=23, name='pack1'}, {slot=24, name='pack2'}, {slot=25, name='pack3'}, {slot=26, name='pack4'}, 
        {slot=27, name='pack5'}, {slot=28, name='pack6'}, {slot=29, name='pack7'}, {slot=30, name='pack8'},
        {slot=31, name='pack9'}, {slot=32, name='pack10'}
    }
    
    if mq.TLO.Window('MerchantWnd').Open() then
        mq.TLO.Window('MerchantWnd').DoClose()
    end

    --Check for and open medicine bag in inventory
    for packIndex = 23, 32 do
        local packs = mq.TLO.Me.Inventory(packIndex)

        -- Check if the item is a medicine bag
        if packs and packs.Type() == 'Make Poison' then
            for _, packInfo in ipairs(packSlots) do
                if packInfo.slot == packIndex then
                    bagSlot = packInfo.name
                    bagData = packs
                    break
                end
            end
        end
    end

    if bagSlot == '' then
        bagSlot = 'poisoncrafting table'
    end
    return bagSlot, bagData
end

local function updateCash()
    MyCash = mq.TLO.Me.Cash() / 1000
end

local function checkCWTN()
    if mq.TLO.Plugin('MQ2Rogue').IsLoaded() and mq.TLO.Me.Class.Name() == 'Rogue' then
        useCWTN = true
    else
        useCWTN = false
    end
end

local function checkForage()
    if mq.TLO.Plugin('mq2autoforage').IsLoaded() then
        useForage = true
    else
        useForage = false
    end
end

local function gatedToPOKEvent()
    -- After gating, this turns the MQ2Rogue back on if it was paused
    -- Also continues travel to the poison merchant
    if gatedToPOK then
        gatedToPOK = false
        if useCWTN and mq.TLO.CWTN.Paused() then
            mq.cmd('/squelch /rog pause off')
        end
        mq.cmd('/easyfind nav locyxz -110.91 1414.45 -108.81')
    end
end

local function potLookup(type, effect, level, onlyVendor)
    local results = {}

    for _, pot in ipairs(Potions) do
        if level > 1 then
            if onlyVendor then
                if pot.type == type and pot.effect == effect and pot.level == tonumber(level)-1 and pot.vendor then
                    table.insert(results, pot)
                end
            else
                if pot.type == type and pot.effect == effect and pot.level == tonumber(level)-1 then
                table.insert(results, pot)
                end
            end
        else
            if onlyVendor then
                if pot.type == type and pot.effect == effect and pot.vendor then
                    table.insert(results, pot)
                end
            else
                if pot.type == type and pot.effect == effect then
                table.insert(results, pot)
                end
            end  
        end
    end

    return results
end

local function cursorLookup(cursor)
    local results = {}

    if cursor ~= nil then
        for _, pot in ipairs(Potions) do
            if table.concat(pot.mats):find(cursor) then
                table.insert(results, pot)
            end
        end
        if #results == 0 then
            printf('%s not found in database', cursor)
            print('Please report this if it\'s a bug')
        end
    else
        print('Nothing on cursor')
    end

    return results
end

local function matList(potion)
    local matsCount = {}

    if potion then
        -- Check for potion ingredients as well as quantity
        for _, mat in ipairs(potion.mats) do
            if matsCount[mat] then
                matsCount[mat] = matsCount[mat] + 1
            else
                matsCount[mat] = 1
            end
        end
    else
        print('No potion selected!')
    end

    return matsCount
end

local function cursorItem()
    if mq.TLO.Cursor() then
        local cursor = mq.TLO.Cursor
        local dragItem = mq.FindTextureAnimation("A_DragItem")
        local fgDrawList = ImGui.GetForegroundDrawList()
        local winX, winY = ImGui.GetWindowPos()
        local winW, winH = ImGui.GetWindowSize()
        local mouseX, mouseY = ImGui.GetMousePos()

        if not (winX <= mouseX and mouseX <= winX + winW and
         winY <= mouseY and mouseY <= winY + winH) then
            return
        end

        dragItem:SetTextureCell(cursor.Icon() - 500)
        fgDrawList:AddTextureAnimation(dragItem, ImVec2(mouseX + 10, mouseY + 10), ImVec2(40, 40))
    end
end

-- function to run the window
local function potionMaker(open)
    -- Specify a default position/size
    local main_viewport = imgui.GetMainViewport()
    imgui.SetNextWindowPos(main_viewport.WorkPos.x + 600, main_viewport.WorkPos.y + 20, ImGuiCond.FirstUseEver)

    -- change the window size
    imgui.SetNextWindowSize(600, 300, ImGuiCond.FirstUseEver)

    local show = false
    open, show = imgui.Begin("Poison Crafter", open)

    if not show then
        mq.unevent('gated')
        mq.unevent('madepotion')

        ImGui.End()
        return open
    end

    ImGui.PushItemWidth(ImGui.GetFontSize() * -12);

    -- Main window element area --
    imgui.Text("Make Poison Skill:")
    imgui.SameLine()
    if HaveTrophy then
        ImGui.TextColored(ImVec4(0.0, 1.0, 0.0, 1.0), tostring(MPSkill))
    else
        imgui.Text(tostring(MPSkill))
    end

    imgui.SameLine(175)
    imgui.Text('Plat: ')
    imgui.SameLine()
    imgui.Text(tostring(MyCash))

    imgui.SameLine(410)
    imgui.Text('Use Gate Potion')
    imgui.SameLine()
    local useGatePot = imgui.Checkbox(' ', useGatePot)

    imgui.SameLine(550)
    if ImGui.Button("Go to vendor") then
        GoVendor = true
    end
    imgui.SameLine()
    if ImGui.Button('Stop') then
        mq.cmd('/squelch /nav stop')
    end

    ImGui.Separator()
    -- Potion selection
    imgui.Columns(4)  -- Set up columns

    -- Display types in the first column
    imgui.Text("          Type")
    imgui.Text('\n')
    for i, pot in ipairs(typeAndEffects) do
        local isSelectedType = (i == selectedTypeIndex)
    
        -- Display the type as a selectable label
        if imgui.Selectable(pot.type, isSelectedType) then
            -- Handle selection logic for types here
            selectedTypeIndex = i
            typeSelect = pot.type
        end
    end
    
    imgui.NextColumn()
    
    -- Display effects in the second column for the selected type
    imgui.Text("         Effects")
    imgui.Text('\n')
    if selectedTypeIndex <= #typeAndEffects then
        local selectedEffects = typeAndEffects[selectedTypeIndex].effects
        for i, effect in ipairs(selectedEffects) do
            local isSelectedEffect = (i == selectedEffectIndex)
    
            -- Display the effect as a selectable label
            if imgui.Selectable(effect, isSelectedEffect) then
                -- Handle selection logic for effects here
                selectedEffectIndex = i
                effectSelect = effect
            end
        end
    end

    imgui.NextColumn()

    imgui.Text('            Level')
    imgui.Text('\n')

    levelSelect, LvlChanged = imgui.Combo("  ", levelSelect, potLevel, #potLevel)

    if LvlChanged then
        lookupResults = potLookup(typeSelect, effectSelect, levelSelect, onlyVendor)
    end

    imgui.NextColumn()

    imgui.Text('\n\n')

    if imgui.Button('Lookup') then
        lookupResults = potLookup(typeSelect, effectSelect, levelSelect, onlyVendor)
    end
    imgui.SameLine()
    if imgui.Button('Cursor Lookup') then
        local cursor = mq.TLO.Cursor.Name()
        lookupResults = cursorLookup(cursor)
    end
    if imgui.Button('Reset') then
        selectedTypeIndex = 1
        selectedEffectIndex = 1
        levelSelect = 1
        lookupResults = {}
    end

    onlyVendor = imgui.Checkbox("Only Store bought", onlyVendor)

    imgui.Text('\n')
    imgui.Text('Quantity:    ')
    imgui.SameLine()
    numPot = imgui.InputText('', tostring(numPot))

    if imgui.Button('Buy Mats') then
        if selectedPotion and tonumber(numPot) > 0 then
            for i = 1, #selectedPotion.mats do
                local matName = selectedPotion.mats[i]
            
                -- Check if the string starts with an asterisk
                if string.sub(matName, 1, 1) == '*' then
                    -- Remove the asterisk
                    matName = string.sub(matName, 2)
                end
                if mq.TLO.FindItemCount(matName)() == 0 then
                    printf('\arError: \axMissing %s!', matName)
                    BuyPot = true
                end
            end
        else
            print('No poison selected!')
        end
    end
    imgui.SameLine()
    if imgui.Button('Craft Poison') then
        if MPSkill >= selectedPotion.reqSkill then
            if selectedPotion then
                local doMake = true
                -- Check if have all ingredients
                for i = 1, #selectedPotion.mats do
                    local matName = selectedPotion.mats[i]
                
                    -- Check if the string starts with an asterisk
                    if string.sub(matName, 1, 1) == '*' then
                        -- Remove the asterisk
                        matName = string.sub(matName, 2)
                    end
                    if mq.TLO.FindItemCount(matName)() == 0 then
                        printf('\arError: \axMissing %s!', matName)
                        doMake = false
                    end
                end
                local bagSlot, bagData = getBagData()
                -- Check that mortar and pestle are empty
                if bagData ~= nil then
                    if not useTable and bagData.Items() > 0 then
                        print('Items in Mortar and Pestle!')
                        doMake = false
                    end
                end
                if doMake then
                    PotMade = 0
                    mq.cmd('/popcustom 15 Do not click while program is running!')
                    print('\ayDo not click while program is running!')
                    MakePot = true
                end
            else
                print('No poison selected!')
            end
        else
            printf('Make Poison skill not high enough! Need %d', selectedPotion.reqSkill)
        end
    end
    --[[imgui.Text(' ')
    imgui.SameLine(80)
    DoPause = false
    if not DoPause then
        if imgui.Button('Pause Potion') then
            MakePot = false
            DoPause = true
        end
    elseif DoPause then
        if imgui.Button('Resume Potion') then
            MakePot = true
            DoPause = false
        end
    end]]

    imgui.Text('\n')

    imgui.Columns(1)  -- Reset columns

    ImGui.Separator()
    -- Only display results if any are available
    if lookupResults and #lookupResults > 0 then
        -- Potions found
        imgui.Text('Poisons')
        imgui.Text('\n')
        imgui.Text('Name')
        imgui.SameLine(185)
        imgui.Text('Trivial')
        imgui.SameLine(255)
        imgui.Text('Req. Skill')
        imgui.SameLine(350)
        imgui.Text('Req. Level')
        imgui.SameLine(425)
        imgui.Text('Materials')
        imgui.SameLine(525)
        imgui.Text('Storebought Materials')

        -- Display information about selected potions
        for i, potion in ipairs(lookupResults) do
            local label = potion.name
            local isSelected2 = selectedPotionIndex == i
            if imgui.Selectable(label, isSelected2) then
                selectedPotion = potion
                selectedPotionIndex = i
            end

            local materials = matList(potion)
            local materialInfo = ""
            for mat, count in pairs(materials) do
                materialInfo = materialInfo .. mat .. ": " .. count .. "\n"
            end

            imgui.SameLine(185)
            imgui.Text('   ' .. potion.trivial)

            imgui.SameLine(260)
            if potion.reqSkill > 0 then
                imgui.Text('    ' .. potion.reqSkill)
            end

            imgui.SameLine(360)
            imgui.Text('     ' .. potion.reqLevel)

            imgui.SameLine(445)
            imgui.HelpMarker(materialInfo)

            imgui.SameLine(550)
            local vendor = potion.vendor and 'Yes' or 'No'
            imgui.Text('      ' .. vendor)
        end
    end
    -- End of main window element area --

    -- Required for window elements
    -- Allow dragging items over window
    cursorItem()
    imgui.Spacing()
    imgui.PopItemWidth()
    imgui.End()
    return open
end

checkCWTN()
checkForage()
updateSkill()

mq.event('gated', 'You have entered The Plane of Knowledge#*#', gatedToPOKEvent)

local openGUI = true

ImGui.Register('potionMaker', function()
    openGUI = potionMaker(openGUI)
end)

while openGUI do
    mq.doevents()
    mq.delay(500)
    getTrophy()
    updateSkill()
    updateCash()
    if GoVendor then
        local function goToVendor(zone)
            -- Check if bound in PoK and uses Gate Potion. Otherwise, runs to PoK
            if zone ~= 'poknowledge' then
                if mq.TLO.Me.ZoneBound() == 'The Plane of Knowledge' then
                    if mq.TLO.FindItem('Vial of Swirling Smoke') and useGatePot then
                        if useCWTN then
                            mq.cmd('/squelch /rog pause on')
                        end
                        mq.cmd('/useitem "vial of swirling smoke"')
                        print('Gating to Plane of Knowledge')
                        gatedToPOK = true
                    end
                else
                mq.cmd('/travelto poknowledge @locyxz -110.91 1414.45 -108.81')
                print('Traveling to Loran Thu`Leth in the Plane of Knowledge!')
                end
            else
                mq.cmd('/easyfind nav locyxz -110.91 1414.45 -108.81')
            end
            GoVendor = false
        end
        goToVendor(mq.TLO.Zone.ShortName())
    end
    if BuyPot then       
        local function buyMats(potion, pots)
            local numMats = 0
            local matHave = 0

            local matsCount = matList(potion)
        
            -- How many potions to make
            local multiplier = tonumber(pots)
        
            -- Multiply counts if needed
            for mat, count in pairs(matsCount) do
                matsCount[mat] = count * multiplier
            end
        
            -- Don't buy items we don't need
            for mat, count in pairs(matsCount) do
                local ownedMat = mq.TLO.FindItemCount(mat)()
                numMats = numMats + 1
                if ownedMat > 0 then
                    if matsCount[mat] - ownedMat < 0 then
                        matsCount[mat] = 0
                        matHave = matHave + 1
                    else
                        matsCount[mat] = matsCount[mat] - ownedMat
                    end
                end
            end
         
            if matHave < numMats then
                if mq.TLO.Spawn('loran').Distance() > 20 then
                    mq.cmd('/easyfind nav locyxz -110.91 1414.45 -108.81')
                    print('Going to merchant')
                end
                while mq.TLO.Nav.Active() do
                    mq.delay(1000)
                end
                if not mq.TLO.Window('MerchantWnd').Open() and mq.TLO.Spawn('loran').Distance() <= 20 then
                    mq.cmd('/multiline ; /tar loran; /timed 5 /click right target')
                end

                while not mq.TLO.Window('MerchantWnd').Open() do
                    mq.delay(500)
                end

                while not mq.TLO.Merchant.ItemsReceived() do
                    mq.delay(500)
                end

                -- Select and buy
                for mat, count in pairs(matsCount) do
                    if count > 0 then
                        mq.TLO.Merchant.SelectItem('=' .. mat)
                        mq.delay(750)
                        if mq.TLO.Merchant.SelectedItem() == mat then
                            mq.TLO.Merchant.Buy(count)
                            mq.delay(500)
                        end
                    end
                end
                if mq.TLO.Window('MerchantWnd').Open() then
                    mq.TLO.Window('MerchantWnd').DoClose()
                end

            else
                print('Do not need to purchase any ingredients')
            end
            BuyPot = false
        end
        buyMats(selectedPotion, numPot)
    end
    while MakePot do
        local function doManual(potion, pots)
            if useForage then
                mq.cmd('/squelch /stopforage')
            end
        
            if useCWTN then
                mq.cmd('/squelch /rog pause on')
            end

            local bagSlot, bagData = getBagData()
            local toMake = tonumber(pots)
            local numMade = 0
            local useTable = false

            if bagSlot == 'poisoncrafting table' then
                useTable = true
            end

            if not useTable then
                if bagData.Open() ~= 1 then
                    mq.cmdf('/itemnotify "%s" rightmouseup', bagData.Name())
                end
            
                if mq.TLO.Window('TradeskillWnd').Open() then
                    mq.TLO.Window('TradeskillWnd/COMBW_ExperimentButton').LeftMouseUp()
                end
                mq.delay(500)
                
                for j = 1, toMake do
                    for i = 1, #potion.mats do
                        local matName = potion.mats[i]
                    
                        -- Check if the string starts with an asterisk
                        if string.sub(matName, 1, 1) == '*' then
                            -- Remove the asterisk
                            matName = string.sub(matName, 2)
                        end
            
                        mq.cmdf('/nomodkey /ctrl /itemnotify "%s" leftmouseup', matName)
                        mq.delay(500)   
                        mq.cmdf('/itemnotify in %s %d leftmouseup', bagSlot, i)
                        mq.delay(500)
                        if i == #potion.mats then
                            mq.cmdf('/combine %s', bagSlot)
                            mq.delay(500)
                            if mq.TLO.Cursor.ID() > 0 then
                                mq.cmdf('/autoinv')
                                numMade = numMade + 1
                                mq.delay(500)
                            end
                            printf('Combine: %d/%d', j, numPot)
                            mq.delay(500)
                        end
                    end
                end
            
                if bagData.Open() == 1 then
                    mq.cmdf('/itemnotify "%s" rightmouseup', bagData.Name())
                end
            elseif useTable then -- Target the poisontable and nav there if too far
                mq.cmdf('/squelch /itemtarget %s', bagSlot)
                if mq.TLO.ItemTarget.Distance() > 20 then
                    mq.cmd('/squelch /easyfind nav locyxz -285.55, 806.20, -92.85')
                end
                -- wait until we arrive
                while mq.TLO.Nav.Active() do
                    mq.delay(1000)
                end
                -- if the tradeskill window isn't open, then open it and experiment
                if mq.TLO.ItemTarget.Distance() < 20 then
                    mq.cmdf('/click left item')
                    print('Opening the table')
                end
                while not mq.TLO.Window('TradeskillWnd').Open() do
                    mq.delay(200)
                end
                mq.TLO.Window('TradeskillWnd/COMBW_ExperimentButton').LeftMouseUp()
                mq.delay(500)
                
                for j = 1, toMake do
                    for i = 1, #potion.mats do
                        local matName = potion.mats[i]
                    
                        -- Check if the string starts with an asterisk
                        if string.sub(matName, 1, 1) == '*' then
                            -- Remove the asterisk
                            matName = string.sub(matName, 2)
                        end
            
                        mq.cmdf('/nomodkey /ctrl /itemnotify "%s" leftmouseup', matName)
                        mq.delay(500)   
                        mq.cmdf('/itemnotify enviro%d leftmouseup', i)
                        mq.delay(500)
                        if i == #potion.mats then
                            mq.cmdf('/combine enviro')
                            mq.delay(500)
                            if mq.TLO.Cursor.ID() > 0 then
                                mq.cmdf('/autoinv')
                                numMade = numMade + 1
                                mq.delay(500)
                            end
                            printf('Combine: %d/%d', j, numPot)
                            mq.delay(500)
                        end
                    end
                end
                mq.cmd('/invoke ${Window[ContainerWindow].DoClose}')
            end

            -- Unpause
            if useForage then
                mq.cmd('/squelch /startforage')
            end
        
            if useCWTN then
                mq.cmd('/squelch /rog pause off')
            end

            if numMade == toMake then
                printf('\agSuccess: \axMade all %d %s', toMake, potion.name)
            elseif toMake > numMade and numMade > 0 then
                printf('\ayPartial Success: \axMade %d of %d %s', numMade, toMake, potion.name)
            elseif numMade == 0 then
                printf('\arFailure: \axAll %d %s failed', toMake, potion.name)
            end
            MakePot = false
        end
        doManual(selectedPotion, numPot)
    end
end